@@ -35,10 +35,6 @@ import designate.central.service
 
 LOG = logging.getLogger(__name__)
 
-# FIXME: create mock, do not use cfg
-cfg.CONF.import_opt('storage_driver', 'designate.central',
-                    group='service:central')
-
 
 # TODO(Federico): move this
 
@@ -437,6 +433,32 @@ class CentralServiceTestCase(CentralBasic):
         assert self.service._enforce_record_quota.called
         assert self.service._update_zone_in_storage.called
 
+    def test_create_recordset_checking_DBDeadLock(self):
+        self.service._enforce_recordset_quota = mock.Mock()
+        self.service._enforce_record_quota = mock.Mock()
+        self.service._is_valid_recordset_name = mock.Mock()
+        self.service._is_valid_recordset_placement = mock.Mock()
+        self.service._is_valid_recordset_placement_subzone = mock.Mock()
+        self.service._is_valid_ttl = mock.Mock()
+
+        self.service.storage.create_recordset = mock.Mock(return_value='rs')
+        self.service._update_zone_in_storage = mock.Mock()
+
+        # NOTE(thirose): Since this is a race condition we assume that
+        #  we will hit it if we try to do the operations in a loop 100 times.
+        for num in range(100):
+            recordset = Mock()
+            recordset.name = "b%s".format(num)
+            recordset.obj_attr_is_set.return_value = True
+            recordset.records = [MockRecord()]
+
+            rs, zone = self.service._create_recordset_in_storage(
+                self.context, Mockzone(), recordset
+            )
+            assert not self.service.storage._retry_on_deadlock.called
+            assert self.service._update_zone_in_storage.called
+            assert self.service.storage.create_recordset.called
+
     def test__create_soa(self):
         self.service._create_recordset_in_storage = Mock(
             return_value=(None, None)
@@ -509,8 +531,8 @@ class CentralZoneTestCase(CentralBasic):
     pool__id = '769ca3fc-5924-4a44-8c1f-7efbe52fbd59'
     recordset__id = '9c85d9b0-1e9d-4e99-aede-a06664f1af2e'
     recordset__id_2 = 'dc85d9b0-1e9d-4e99-aede-a06664f1af2e'
-    recordset__id_3 = '121s-de21s-4e99-aede-a06664f1af2e'
-    zone_export__id = 'de21s-4e99-4424-b77f-2dd0179fa041'
+    recordset__id_3 = '2a94a9fe-30d1-4a15-9071-0bb21996d971'
+    zone_export__id = 'e887597f-9697-47dd-a202-7a2711f8669c'
 
     def setUp(self):
         super(CentralZoneTestCase, self).setUp()
@@ -519,6 +541,12 @@ class CentralZoneTestCase(CentralBasic):
             if d['name'] not in ('org',):
                 raise exceptions.TldNotFound
 
+        def storage_find_tlds(c):
+            return objects.TldList.from_list(
+                [objects.Tld.from_dict({'name': 'org'})]
+            )
+
+        self.service.storage.find_tlds = storage_find_tlds
         self.service.storage.find_tld = storage_find_tld
 
     def test__is_valid_zone_name_valid(self):
